@@ -23,6 +23,7 @@ import fcntl
 import functools
 import libevdev
 import os
+import threading
 
 try:
     import pyudev
@@ -36,7 +37,7 @@ from hidtools.uhid import UHIDDevice
 from hidtools.util import BusType
 
 from pathlib import Path
-from typing import Any, ClassVar, Dict, List, Optional, Tuple, Type, Union
+from typing import Any, ClassVar, Optional, Type, Union
 
 logger = logging.getLogger("hidtools.device.base_device")
 
@@ -119,10 +120,10 @@ class HIDIsReady(object):
     See :meth:`is_ready` for details.
     """
 
-    def __init__(self: "HIDIsReady", uhid: UHIDDevice) -> None:
+    def __init__(self, uhid: UHIDDevice) -> None:
         self.uhid = uhid
 
-    def is_ready(self: "HIDIsReady") -> HidReadiness:
+    def is_ready(self) -> HidReadiness:
         """
         Overwrite in subclasses: should return True or False whether
         the attached uhid device is ready or not.
@@ -133,9 +134,9 @@ class HIDIsReady(object):
 class UdevHIDIsReady(HIDIsReady):
     _pyudev_context: ClassVar[Optional[pyudev.Context]] = None
     _pyudev_monitor: ClassVar[Optional[pyudev.Monitor]] = None
-    _uhid_devices: ClassVar[Dict[int, HidReadiness]] = {}
+    _uhid_devices: ClassVar[dict[int, HidReadiness]] = {}
 
-    def __init__(self: "UdevHIDIsReady", uhid: UHIDDevice) -> None:
+    def __init__(self, uhid: UHIDDevice) -> None:
         super().__init__(uhid)
         self._init_pyudev()
 
@@ -172,7 +173,7 @@ class UdevHIDIsReady(HIDIsReady):
 
             readiness.is_ready = ready
 
-    def is_ready(self: "UdevHIDIsReady") -> HidReadiness:
+    def is_ready(self) -> HidReadiness:
         try:
             return self._uhid_devices[self.uhid.hid_id]
         except KeyError:
@@ -181,19 +182,19 @@ class UdevHIDIsReady(HIDIsReady):
 
 class EvdevMatch(object):
     def __init__(
-        self: "EvdevMatch",
+        self,
         *,
-        requires: List[Any] = [],
-        excludes: List[Any] = [],
-        req_properties: List[Any] = [],
-        excl_properties: List[Any] = [],
+        requires: list[Any] = [],
+        excludes: list[Any] = [],
+        req_properties: list[Any] = [],
+        excl_properties: list[Any] = [],
     ) -> None:
         self.requires = requires
         self.excludes = excludes
         self.req_properties = req_properties
         self.excl_properties = excl_properties
 
-    def is_a_match(self: "EvdevMatch", evdev: libevdev.Device) -> bool:
+    def is_a_match(self, evdev: libevdev.Device) -> bool:
         for m in self.requires:
             if not evdev.has(m):
                 return False
@@ -217,7 +218,7 @@ class EvdevDevice(object):
     and properties.
     """
 
-    def __init__(self: "EvdevDevice", sysfs: Path) -> None:
+    def __init__(self, sysfs: Path) -> None:
         self.sysfs = sysfs
         self.event_node: Any = None
         self.libevdev: Optional[libevdev.Device] = None
@@ -234,17 +235,17 @@ class EvdevDevice(object):
         self.open()
 
     @property
-    def name(self: "EvdevDevice") -> str:
+    def name(self) -> str:
         assert "NAME" in self.uevents
 
         return self.uevents["NAME"]
 
     @property
-    def evdev(self: "EvdevDevice") -> Path:
+    def evdev(self) -> Path:
         return Path("/dev/input") / self.sysfs.name
 
     def matches_application(
-        self: "EvdevDevice", application: str, matches: Dict[str, EvdevMatch]
+        self, application: str, matches: dict[str, EvdevMatch]
     ) -> bool:
         if self.libevdev is None:
             return False
@@ -257,7 +258,7 @@ class EvdevDevice(object):
         )
         assert False  # hid-tools likely needs an update
 
-    def open(self: "EvdevDevice") -> libevdev.Device:
+    def open(self) -> libevdev.Device:
         self.event_node = open(self.evdev, "rb")
         self.libevdev = libevdev.Device(self.event_node)
 
@@ -269,7 +270,7 @@ class EvdevDevice(object):
 
         return self.libevdev
 
-    def close(self: "EvdevDevice") -> None:
+    def close(self) -> None:
         if self.libevdev is not None and self.libevdev.fd is not None:
             self.libevdev.fd.close()
             self.libevdev = None
@@ -281,7 +282,7 @@ class EvdevDevice(object):
 class BaseDevice(UHIDDevice):
     # default _application_matches that matches nothing. This needs
     # to be set in the subclasses to have get_evdev() working
-    _application_matches: Dict[str, EvdevMatch] = {}
+    _application_matches: dict[str, EvdevMatch] = {}
 
     def __init__(
         self,
@@ -294,6 +295,7 @@ class BaseDevice(UHIDDevice):
         self._kernel_is_ready: HIDIsReady = UdevHIDIsReady(self)
         if rdesc_str is None and rdesc is None:
             raise Exception("Please provide at least a rdesc or rdesc_str")
+        self._input_nodes: Optional[list[EvdevDevice]] = None
         super().__init__()
         if name is None:
             name = f"uhid gamepad test {self.__class__.__name__}"
@@ -305,7 +307,6 @@ class BaseDevice(UHIDDevice):
         self.opened = False
         self.started = False
         self.application = application
-        self._input_nodes: Optional[list[EvdevDevice]] = None
         if rdesc is None:
             assert rdesc_str is not None
             self.rdesc = hid.ReportDescriptor.from_human_descr(rdesc_str)  # type: ignore
@@ -313,7 +314,7 @@ class BaseDevice(UHIDDevice):
             self.rdesc = rdesc  # type: ignore
 
     @property
-    def power_supply_class(self: "BaseDevice") -> Optional[PowerSupply]:
+    def power_supply_class(self) -> Optional[PowerSupply]:
         ps = self.walk_sysfs("power_supply", "power_supply/*")
         if ps is None or len(ps) < 1:
             return None
@@ -321,7 +322,7 @@ class BaseDevice(UHIDDevice):
         return PowerSupply(ps[0])
 
     @property
-    def led_classes(self: "BaseDevice") -> List[LED]:
+    def led_classes(self) -> list[LED]:
         leds = self.walk_sysfs("led", "**/max_brightness")
         if leds is None:
             return []
@@ -329,25 +330,43 @@ class BaseDevice(UHIDDevice):
         return [LED(led.parent) for led in leds]
 
     @property
-    def kernel_is_ready(self: "BaseDevice") -> bool:
+    def kernel_is_ready(self) -> bool:
         return self._kernel_is_ready.is_ready().is_ready and self.started
 
     @property
-    def kernel_ready_count(self: "BaseDevice") -> int:
+    def kernel_ready_count(self) -> int:
         return self._kernel_is_ready.is_ready().count
 
     @property
-    def input_nodes(self: "BaseDevice") -> List[EvdevDevice]:
+    def input_nodes(self) -> list[EvdevDevice]:
         if self._input_nodes is not None:
             return self._input_nodes
 
         if not self.kernel_is_ready or not self.started:
             return []
 
+        # Starting with kernel v6.16, an event is emitted when
+        # userspace opens a kernel device, and for some devices
+        # this translates into a SET_REPORT.
+        # Because EvdevDevice(path) opens every single evdev node
+        # we need to have a separate thread to process the incoming
+        # SET_REPORT or we end up having to wait for the kernel
+        # timeout of 5 seconds.
+        done = False
+
+        def dispatch():
+            while not done:
+                self.dispatch(1)
+
+        t = threading.Thread(target=dispatch)
+        t.start()
+
         self._input_nodes = [
             EvdevDevice(path)
             for path in self.walk_sysfs("input", "input/input*/event*")
         ]
+        done = True
+        t.join()
         return self._input_nodes
 
     def match_evdev_rule(self, application, evdev):
@@ -390,11 +409,11 @@ class BaseDevice(UHIDDevice):
         return []
 
     @property
-    def application_matches(self: "BaseDevice") -> Dict[str, EvdevMatch]:
+    def application_matches(self) -> dict[str, EvdevMatch]:
         return self._application_matches
 
     @application_matches.setter
-    def application_matches(self: "BaseDevice", data: Dict[str, EvdevMatch]) -> None:
+    def application_matches(self, data: dict[str, EvdevMatch]) -> None:
         self._application_matches = data
 
     def get_evdev(self, application=None):
@@ -416,6 +435,8 @@ class BaseDevice(UHIDDevice):
                     if self.match_evdev_rule(application, _evdev.libevdev):
                         return _evdev.libevdev
 
+        return None
+
     def is_ready(self):
         """Returns whether a UHID device is ready. Can be overwritten in
         subclasses to add extra conditions on when to consider a UHID
